package javax.smartcardio;

import java.util.Collections;
import java.util.List;
import java.security.Security;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;
import sun.security.jca.GetInstance;
import java.security.Provider;

public final class TerminalFactory
{
    private static final String PROP_NAME = "javax.smartcardio.TerminalFactory.DefaultType";
    private static final String defaultType;
    private static final TerminalFactory defaultFactory;
    private final TerminalFactorySpi spi;
    private final Provider provider;
    private final String type;
    
    private TerminalFactory(TerminalFactorySpi spi, Provider provider, String type) {
        this.spi = spi;
        this.provider = provider;
        this.type = type;
    }
    
    public static String getDefaultType() {
        return TerminalFactory.defaultType;
    }
    
    public static TerminalFactory getDefault() {
        return TerminalFactory.defaultFactory;
    }
    
    public static TerminalFactory getInstance(String algorithm, Object param) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("TerminalFactory", TerminalFactorySpi.class, algorithm, param);
        return new TerminalFactory((TerminalFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public static TerminalFactory getInstance(String algorithm, Object param, String provider) throws NoSuchAlgorithmException, NoSuchProviderException {
        GetInstance.Instance instance = GetInstance.getInstance("TerminalFactory", TerminalFactorySpi.class, algorithm, param, provider);
        return new TerminalFactory((TerminalFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public static TerminalFactory getInstance(String algorithm, Object param, Provider provider) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("TerminalFactory", TerminalFactorySpi.class, algorithm, param, provider);
        return new TerminalFactory((TerminalFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public Provider getProvider() {
        return this.provider;
    }
    
    public String getType() {
        return this.type;
    }
    
    public CardTerminals terminals() {
        return this.spi.engineTerminals();
    }
    
    @Override
    public String toString() {
        return "TerminalFactory for type " + this.type + " from provider " + this.provider.getName();
    }
    
    static {
        String trim = ((String)AccessController.doPrivileged(new GetPropertyAction("javax.smartcardio.TerminalFactory.DefaultType", "PC/SC"))).trim();
        Object defaultFactory2 = null;
        try {
            defaultFactory2 = getInstance(trim, null);
        }
        catch (Exception ex) {}
        if (defaultFactory2 == null) {
            try {
                trim = "PC/SC";
                Provider provider = Security.getProvider("SunPCSC");
                if (provider == null) {
                    provider = (Provider)Class.forName("sun.security.smartcardio.SunPCSC").newInstance();
                }
                defaultFactory2 = getInstance(trim, null, provider);
            }
            catch (Exception ex2) {}
        }
        if (defaultFactory2 == null) {
            trim = "None";
            defaultFactory2 = new TerminalFactory(TerminalFactory.NoneFactorySpi.INSTANCE, TerminalFactory.NoneProvider.INSTANCE, "None");
        }
        defaultType = trim;
        defaultFactory = defaultFactory2;
    }
    
    private static final class NoneProvider extends Provider
    {
        private static final long serialVersionUID = 2745808869881593918L;
        static final Provider INSTANCE;
        
        private NoneProvider() {
            super("None", 1.0, "none");
        }
        
        static {
            INSTANCE = new TerminalFactory.NoneProvider();
        }
    }
    
    private static final class NoneFactorySpi extends TerminalFactorySpi
    {
        static final TerminalFactorySpi INSTANCE;
        
        @Override
        protected CardTerminals engineTerminals() {
            return TerminalFactory.NoneCardTerminals.INSTANCE;
        }
        
        static {
            INSTANCE = new TerminalFactory.NoneFactorySpi();
        }
    }
    
    private static final class NoneCardTerminals extends CardTerminals
    {
        static final CardTerminals INSTANCE;
        
        @Override
        public List<CardTerminal> list(CardTerminals.State state) throws CardException {
            if (state == null) {
                throw new NullPointerException();
            }
            return Collections.emptyList();
        }
        
        @Override
        public boolean waitForChange(long n) throws CardException {
            throw new IllegalStateException("no terminals");
        }
        
        static {
            INSTANCE = new TerminalFactory.NoneCardTerminals();
        }
    }
}
